/**
 * 
 */
package com.platomix.datasafe.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.platomix.datasafe.enums.TaskStateEnum;
import com.platomix.datasafe.model.EventDecode;
import com.platomix.datasafe.model.TaskInfo;
import com.platomix.datasafe.service.RsaKeyService;
import com.platomix.datasafe.service.TaskInfoService;
import com.platomix.datasafe.support.Consts;
import com.platomix.datasafe.support.FileKit;
import com.platomix.datasafe.support.R;
import com.platomix.datasafe.support.RsaUtil;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.lang.UUID;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("/api")
public class ApiController {

	@Autowired
	private TaskInfoService taskInfoService;
	
	@Autowired
	private RsaKeyService rsaKeyService;

	@Autowired
	private ApplicationContext applicationContext;

	/**
	 * 检测任务是否已完成
	 * 
	 * @param taskId
	 * @return
	 */
	@GetMapping("/task/check/{taskId}")
	public R<String> checkTask(@PathVariable Long taskId) {
		TaskInfo taskInfo = taskInfoService.getById(taskId);
		if (taskInfo == null) {
			return R.fail(404, "任务不存在");
		}
		if (taskInfo.getState() == TaskStateEnum.FAILURE) {
			return R.fail(201, "任务处理失败");
		}
		if (taskInfo.getState() == TaskStateEnum.SUCCESS) {
			return R.ok("任务已完成");
		}
		return R.fail("任务正在进行中");
	}

	/**
	 * 实时解密加密手机号 - 有数量限制({@link Consts#DEFAULT_DATA_SIZE})
	 * 
	 * @param cipherTxts
	 * @return
	 */
	@PostMapping("/mobile/decode")
	public List<String> decode(@RequestBody List<String> dataList) {
		String key = rsaKeyService.getKey();
		List<String> resultList = new ArrayList<String>();
		for(int i = 0; i < dataList.size() && i < Consts.DEFAULT_DATA_SIZE; i++) {
			String mobile = RsaUtil.decrypt(key, dataList.get(i));
			if(mobile != null) {
				resultList.add(mobile);
			}
		}
		//乱序
		Collections.shuffle(resultList);
		return resultList;
	}
	
	/**
	 * 批量解析文件并下载
	 * @param file
	 * @param request
	 * @param response
	 */
	@SuppressWarnings("rawtypes")
	@PostMapping("/mobile/batchDecode")
	public ResponseEntity batchDecode(@RequestParam("txtfile") MultipartFile file, 
			HttpServletRequest request, HttpServletResponse response) {
		String channel = request.getParameter(Consts.CHANNEL_PARAM_NAME);
		log.info("上传文件,创建任务|fileName={},fileSize={},channel={}", file.getOriginalFilename(), file.getSize(), channel);
		// 1.判断文件类型
		if (file == null || file.isEmpty()) {
			log.error("文件有问题");
			return ResponseEntity.badRequest().build();
		}
		if (!file.getOriginalFilename().endsWith(".txt")) {
			log.error("必须是txt文件");
			return ResponseEntity.badRequest().build();
		}
		String downloadFilePath = Consts.getUploadDir() + UUID.fastUUID().toString(true) + ".txt";
		File downloadFile = new File(downloadFilePath);
		try {
			long startTime = System.currentTimeMillis();
			//自定义缓冲大小，8M缓冲
			IoUtil.copy(file.getInputStream(), new FileOutputStream(downloadFile, true), 2 << 12);
			log.info("文件接收完成|cost:{}ms", System.currentTimeMillis() - startTime);
			// 2.解密文件，并存储到文件夹
			String key = rsaKeyService.getKey();
			String destFilePath = Consts.getDownloadDir() + UUID.fastUUID().toString(true) + ".txt";
			FileKit.decodeFile(downloadFilePath, destFilePath, key);
			// 3.下载文件
			HttpHeaders httpHeaders = new HttpHeaders();
			String fileName = URLEncoder.encode(channel + "_mobile_" + DateUtil.format(new Date(), "yyyyMMddHHmmss"), "UTF-8");
	        //指定下载后的文件名等，new String(filename.getBytes("UTF-8"),"ISO8859-1")很重要，不然会下载中文名的时候名字为空
	        httpHeaders.add("Content-Disposition","attachment; filename="+fileName + ".txt");
	        File destFile = new File(destFilePath);
			return ResponseEntity.ok()
					.headers(httpHeaders)
					.contentLength(destFile.length())
					.contentType(MediaType.parseMediaType("application/octet-stream"))
					.body(new InputStreamResource(new FileInputStream(destFile)));
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return ResponseEntity.badRequest().build();
	}

	/**
	 * 异步解密手机号(基于文件) - 创建任务
	 * 
	 * @return
	 */
	@PostMapping("/task/create")
	public R<Long> createTask(@RequestParam("txtfile") MultipartFile file, HttpServletRequest request) {
		String channel = request.getParameter(Consts.CHANNEL_PARAM_NAME);
		log.info("上传文件,创建任务|fileName={},fileSize={},channel={}", file.getOriginalFilename(), file.getSize(), channel);
		// 1.判断文件类型
		if (file == null || file.isEmpty()) {
			return R.fail("文件有问题");
		}
		if (!file.getOriginalFilename().endsWith(".txt")) {
			return R.fail("必须是txt文件");
		}
		String srcFileName = file.getOriginalFilename();
		File downloadFile = new File(Consts.getUploadDir() + UUID.fastUUID().toString(true) + ".txt");
		try {
			long startTime = System.currentTimeMillis();
			//自定义缓冲大小，8M缓冲
			IoUtil.copy(file.getInputStream(), new FileOutputStream(downloadFile, true), 2 << 12);
			log.info("文件接收完成|cost:{}ms", System.currentTimeMillis() - startTime);
			// 2.存储文件到文件夹 使用4M缓冲
//			StreamUtils.copy(file.getInputStream(), new FileOutputStream(downloadFile, true));
			// 创建任务
			TaskInfo taskInfo = new TaskInfo();
			taskInfo.setSrcFileSize(file.getSize());
			taskInfo.setChannel(channel);
			taskInfo.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
			taskInfo.setSrcFileName(srcFileName);
			taskInfo.setSrcFilePath(downloadFile.getAbsolutePath());
			taskInfo.setState(TaskStateEnum.DOING);
			boolean ret = taskInfoService.save(taskInfo);
			if (ret) {
				// 事件通知
				applicationContext
						.publishEvent(new EventDecode(this, taskInfo.getTaskId(), downloadFile.getAbsolutePath()));
				return R.ok(taskInfo.getTaskId());
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return R.fail("上传文件失败，请联系工作人员检查");
	}
	
	/**
	 * 下载文件
	 * 
	 * @param taskId
	 * @throws Exception
	 */
	@GetMapping("/download/{taskId}")
	public void download(HttpServletRequest request, HttpServletResponse response,
			@PathVariable Long taskId) throws Exception {
		TaskInfo taskInfo = taskInfoService.getById(taskId);
		if (taskInfo == null || taskInfo.getTaskId() == null) {
			log.warn("任务不存在|taskId={}", taskId);
			return;
		}
		if (taskInfo.getState() != TaskStateEnum.SUCCESS) {
			log.warn("任务未完成,不能下载|taskId={}", taskId);
			return;
		}
		File downloadFile = new File(taskInfo.getDestFilePath());
		if (!downloadFile.exists()) {
			log.error("任务已完成，但是下载文件不存在，请检查|taskid={}", taskId);
			return;
		}
		response.setContentType("application/octet-stream");
		response.setContentLength((int) downloadFile.length());
		String channel = request.getParameter(Consts.CHANNEL_PARAM_NAME);
		String fileName = URLEncoder.encode(channel + "_mobile_" + DateUtil.format(new Date(), "yyyyMMddHHmmss"), "UTF-8");
		response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".txt");
		try (FileInputStream fileInputStream = new FileInputStream(downloadFile)) {
			//数据Copy，默认缓存8192
			IoUtil.copy(fileInputStream, response.getOutputStream());
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
	}
}
